/* $Id: AbstractMapPojoPathFunction.java 517 2008-04-02 19:58:21Z hohwille $
 * Copyright (c) The m-m-m Team, Licensed under the Apache License, Version 2.0
 * http://www.apache.org/licenses/LICENSE-2.0 */
package net.sf.mmm.util.pojo.path.base;

import java.util.Map;

import net.sf.mmm.util.pojo.path.api.PojoPathContext;

/**
 * This is the abstract base implementation for a
 * {@link net.sf.mmm.util.pojo.path.api.PojoPathFunction} that operates
 * on a {@link Map}.
 * 
 * @param <VALUE> is the value this function traverses to starting from the
 *        actual POJO.
 * 
 * @author Joerg Hohwiller (hohwille at users.sourceforge.net)
 */
public abstract class AbstractMapPojoPathFunction<VALUE> extends
    AbstractDeterministicPojoPathFunction<Map<String, Object>, VALUE> {

  /**
   * {@inheritDoc}
   */
  @SuppressWarnings("unchecked")
  public VALUE get(Map<String, Object> current, String functionName, PojoPathContext context) {

    return (VALUE) current.get(functionName);
  }

  /**
   * {@inheritDoc}
   */
  @SuppressWarnings("unchecked")
  public VALUE set(Map<String, Object> current, String functionName, VALUE value,
      PojoPathContext context) {

    return (VALUE) current.put(functionName, value);
  }

}
